/*
 * Copyright (c) 2009 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.app.views.layercomp;

import ch.kuramo.javie.app.views.LayerCompositionView;
import ch.kuramo.javie.core.Layer;

public class CameraTransformElement extends Element {

	private Element[] children;


	public CameraTransformElement(LayerElement parent) {
		super(parent);
	}

	public String getColumnText(int columnIndex) {
		switch (columnIndex) {
			case LayerCompositionView.NAME_COL:
				return "トランスフォーム";

			default:
				return null;
		}
	}

	@Override
	public boolean hasChildren() {
		return true;
	}

	@Override
	public Element[] getChildren() {
		if (children == null) {
			Layer layer = ((LayerElement) parent).layer;
			children = new Element[] {
					new LayerAnimatableValueElement(this, layer, "pointOfInterest", "目標点"),
					new LayerAnimatableValueElement(this, layer, "position", "位置"),
					new LayerAnimatableValueElement(this, layer, "orientation", "方向", "°"),
					new LayerAnimatableValueElement(this, layer, "rotationX", "X 回転", "°"),
					new LayerAnimatableValueElement(this, layer, "rotationY", "Y 回転", "°"),
					new LayerAnimatableValueElement(this, layer, "rotationZ", "Z 回転", "°")
			};
		}
		return children;
	}

}
